//---------------------------------------------------------------------------
// Copyright (C) 1997-2007 iNTERFACEWARE Inc.  All Rights Reserved
//
// Module: XML
//
// Description:
//
// This example application exercises Chameleon's HL7 <-> XML conversion API.
//
// Author: iNTERFACEWARE
// Revision: $Revision: 1.2 $
//
//---------------------------------------------------------------------------
#include <CHM\CHMpreCompHead.h>
#pragma hdrstop

#include <iostream.h>
#include <CHM/CHMllpServer.h>

// This Example Server class is defined so that the application can utilize the
// MakeMessagePrintable function (to format HL7 messages for readibility), which
// is defined by CHMllpServer. Note that the Example Server is not vital to the
// application. The user is free to utilize their own formatting implementation.
class ExampleServer
 : public CHMllpServer
{
public:
   ExampleServer() {}
   virtual ~ExampleServer() {}

   virtual void OnErrorReceived(const char* ErrorMessage, CHMint32 Code) {}
   virtual void OnClientConnected(CHMllpConnection& Client) {}
   virtual void OnClientDisconnected(CHMllpConnection& Client) {}
   virtual void OnClientErrorReceived(CHMllpConnection& Client, const char* ErrorMessage, CHMint32 ErrorCode) {}  
   virtual void OnMessageReceived(CHMllpConnection& Client, const char* Message) {}
   virtual void OnDataIgnored(CHMllpConnection& Client, const char* Data) {}
};

int main()
{
   try
   {
      CHMengine Engine;
      ExampleServer Server;

      Engine.Load("project.vmd");
      Engine.SetLog(new CHMcoutOutputLog);

      Engine.OutputLine("We start with a flatwire message:");
      CHMstring sFlatWire;
      sFlatWire = "MSH|^~\\&|REGADT|MCM|RSP1P8|MCM|199601051530|SEC|ADT^A44|00000007|P|2.3\r";
      sFlatWire = sFlatWire + "EVN|A44|199601051530\r";
      sFlatWire = sFlatWire + "PID|||MR2^^^XYZ||JONES^WILLIAM^A^JR||19501010|M|||123 EAST STREET^^NY^NY^10021||(212)111-3333|||S||ACCT1\r";
      sFlatWire = sFlatWire + "MRG|MR1^^^XYZ||ACCT1\r";
      sFlatWire = sFlatWire + "MRG|MR1^^^XYZ||ACCT1\r";
      sFlatWire = sFlatWire + "MRG|MR1^^^XYZ||ACCT1\r";

      const char* FlatWire = sFlatWire.c_str();
      Engine.OutputLine(Server.MakeMessagePrintable(FlatWire, "\r\n", sFlatWire.size()));

      // The translateMessageToXml method uses the XML Translation Scheme specified
      // in the Options -> XML settings of the message definition file.
      // Read the XML <-> HL7 Translation chapter of the manual for more information.
      const char* XmlFormat = Engine.translateMessageToXml(FlatWire);     
      Engine.OutputLine("Translating to XML:");
      Engine.OutputLine(XmlFormat);
      
      // Same XML Translation Scheme as for translateMessageToXml above
      const char* Hl7 = Engine.translateXmlToMessage(XmlFormat);
      Engine.OutputLine("And translating back again...");
      Engine.OutputLine(Server.MakeMessagePrintable(Hl7, "\r\n", strlen(Hl7))); 
   }
   catch (CHMerror& Error)
   {
      cerr << Error.Description();
   }

   cout << "Press Enter to Exit..." << endl;
   getchar();
   
   return 0;
}
