Imports iNTERFACEWARE.Chameleon
Imports iNTERFACEWARE.Iguana.Plugin

Module IguanaPlugin
   ' The DestinationPluginExample is an example of an Iguana destination plugin.
   Class DestinationPluginExample
      Implements Log

      Public Sub New()
         ' The first step in making the generated PrefixTestEngine object useable 
         ' is to load the message definition or 'vmd' file. This file contains
         ' the information about the structure of the messages, tables, and 
         ' data mappings that were defined in the Chameleon GUI. 
         '  
         ' PrefixTestEngine is a generated engine which means that it has an interface
         ' based on the Messages and Tables you define in your vmd file. The 
         ' use of stub code is helpful since it means that the compiler can 
         ' catch typos or spelling mistakes of fields at compile rather than 
         ' run time. However, it is not the only way you can use Chameleon. It
         ' is possible to use a non-stubcode interface for the Chameleon
         ' engine which we call the 'dynamic interface'. Please review the 
         ' manual documentation for more details if you need this functionality
         ' - most customers find that the stub code interface works well for 
         ' their applications.

         engine.AddLog(Me)
         System.Console.WriteLine("Loading test.vmd...")
         engine.Load("test.vmd")
         engine.CurrentConfigIndex = 0
         System.Console.WriteLine("test.vmd loaded")
      End Sub

      ' Log interface implementation.
      Sub Println(ByVal line As String) Implements Log.Println
         System.Console.WriteLine(line)
         logString += line
         logString += "\n"
      End Sub
      Public Sub ProcessMessage(ByVal inputs As DestinationMessageReceivedEventInputs, _
                                ByVal outputs As DestinationMessageReceivedEventOutputs, _
                                ByVal resubmissionPreview As Boolean)

         Try
            ' The resubmissionPreview flag indicates whether or not we're processing a resubmission 
            ' preview message.  When processing a resubmission preview message, the plugin should 
            ' simulate the processing that would happen for a regular message, but without causing 
            ' external side effects.  For example, no database commits should be made for resubmission
            ' preview messages.  In this example, regular message processing has no external side 
            ' effects, so we simply ignore the resubmissionPreview flag.

            System.Console.WriteLine("")
            System.Console.WriteLine("*** Message Received ***")
            System.Console.WriteLine(inputs.Message.Replace("\r", "\n"))

            ' Notice that we 'manually' invoke the parseMessage method on the engine 
            ' object. This will fire one of the generated onMessage events which are 
            ' defined in the DestinationPluginEngine class.
            ' 
            ' The plugin classes are not tightly coupled to the HL7Engine object. 
            ' You can use any kind of transport you like with  Chameleon, including 
            ' your own transports or third party components.
            ' 
            ' In this application we do not block on events - this is because the
            ' plugin components are asynchronous. You can write synchronous 
            ' applications using Chameleon but you would need to use another 
            ' transport mechanism other than the plugin components supplied here.
            logString = ""
            engine.ParseMessage(inputs.Message, Nothing)

            ' Tell Iguana that the message has been successfully processed.
            outputs.MessageProcessedFlag = True

         Catch ex As Exception
            System.Console.WriteLine(ex.ToString())

            ' Tell Iguana that the message was not successfully processed.
            outputs.MessageProcessedFlag = False

            ' Add an error-level log message to Iguana's logs describing the failure.
            outputs.AddErrorLogMessage(ex.ToString())

            ' You can use ResendMessageFlag and ResendMessageDelay to request that Iguana
            ' resend the same message again after a delay period.  This is useful in situations
            ' where the message processing failure was due to a time sensitive cause, and
            ' message processing might succeed at a later time.
            '
            ' outputs.ResendMessageFlag = true;
            ' outputs.ResendMessageDelay = 1000; ' Resend after 1 second
            '
            ' You can use StopChannelFlag, StopChannelErrorFlag, and StopChannelReason to 
            ' stop the Iguana  channel with or without an error flag, and provide a reason 
            ' for the stop request.
            ' 
            ' outputs.StopChannelFlag = true;
            ' outputs.StopChannelErrorFlag = true;
            ' outputs.StopChannelReason = "An error occurred.";
         End Try

         ' You can send additional error/warning/info/debug log messages to Iguana.
         ' In this case we simply send an info-level log message containing the 
         ' Chameleon engine's log output.
         If (logString.Length > 0) Then
            outputs.AddInfoLogMessage(logString)
         End If
      End Sub

      ' This event is triggered when Iguana sends a message.
      Public Sub OnMessageReceived(ByVal sender As Object, _
                                   ByVal args As System.EventArgs, _
                                   ByVal inputs As DestinationMessageReceivedEventInputs, _
                                   ByVal outputs As DestinationMessageReceivedEventOutputs) Handles destination.MessageReceived

         ' Note that exceptions can never be thrown outside the handler.
         Try
            ' This event is the main entry point for where we get an actual HL7 
            ' message from Iguana. 

            ProcessMessage(inputs, outputs, False)

         Catch ex As Exception
            System.Console.WriteLine(ex.ToString())
         End Try
      End Sub
      ' This event is triggered when Iguana sends a resubmission preview message.
      Public Sub OnResubmissionPreviewMessageReceived(ByVal sender As Object, _
                                                     ByVal args As System.EventArgs, _
                                                     ByVal inputs As DestinationMessageReceivedEventInputs, _
                                                     ByVal outputs As DestinationMessageReceivedEventOutputs) Handles destination.ResubmissionPreviewMessageReceived

         ' Note that exceptions can never be thrown outside the handler.
         Try
            ' This handler should simulate the processing that would be performed by the
            ' OnMessageReceived event handler without causing external side effects.
            ' For example, database changes should not be made for resubmission preview messages.

            ProcessMessage(inputs, outputs, True)

         Catch ex As Exception
            System.Console.WriteLine(ex.ToString())
         End Try
      End Sub

      ' This event is triggered when the plugin disconnects from Iguana.
      Public Sub OnDisconnected(ByVal sender As Object, ByVal args As System.EventArgs) Handles destination.Disconnected
         ' Note that exceptions can never be thrown outside the handler.
         Try
            System.Console.WriteLine("Source disconnected.")
         Catch ex As Exception
            System.Console.WriteLine(ex.ToString())
         End Try
      End Sub

      Public WithEvents destination As New Destination
      Private engine As New DestinationPluginEngine
      Private logString As New String("")
   End Class

   ' The main entry point for the application.
   Sub Main(ByVal args() As String)
      Try
         If args.Length < 3 Then
            Throw New Exception("Invalid command line arguments.")
         End If

         Dim hostname As String = args(0)
         Dim port As Integer = Integer.Parse(args(1))
         Dim channel As String = args(2)

         Dim destinationPluginExample As New DestinationPluginExample
         destinationPluginExample.destination.Connect(hostname, port, channel)

         ' Wait for the plugin to disconnect.
         ' Could also use waitForDisconnectWithTimeout() here.
         destinationPluginExample.destination.WaitForDisconnect()

         ' Print the reason for disconnection.
         Dim disconnectReason As DisconnectReason = destinationPluginExample.destination.DisconnectReason
         System.Console.WriteLine(disconnectReason.Description)
      Catch ex As Exception
         System.Console.WriteLine(ex.ToString())
         System.Console.WriteLine()
         System.Console.WriteLine("This is an Iguana destination plugin.  It runs as a command line application")
         System.Console.WriteLine("which connects via TCP/IP to the plugin communication server within Iguana.")
         System.Console.WriteLine("It will be necessary to run a channel in Iguana and set up a channel with")
         System.Console.WriteLine("a To Plugin component for this to connect to.  See:")
         System.Console.WriteLine()
         System.Console.WriteLine("   http://www.interfaceware.com/manual/plugin_setup_vbnet.html")
         System.Console.WriteLine()
         System.Console.WriteLine("Once the channel is set up and running you can connect to it with this usage:")
         System.Console.WriteLine()
         System.Console.WriteLine("  DestinationPluginExample <iguana hostname> <plugin communication port> <channel name>")
         System.Console.WriteLine()
         System.Console.WriteLine("The plugin communication port is the port that Iguana listens on for plugin")
         System.Console.WriteLine("communication.")
      End Try
   End Sub
End Module
